package javaballsopt;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;

public class JavaFXTest extends Application 
{
	
	public static void main(String[] args) 
	{
		launch(args);	   
	}

	@Override
	public void start(Stage stage) 
	{
		try
		{
			// get the image
			URL url = javaballsopt.JavaBallsApplet.class.getResource("resources/ball.png");
			Image lImage = new Image(url.openStream());

			// create the ball object
			final JavaFXBallBitmap lJavaFXBall = new JavaFXBallBitmap(lImage);
			//final JavaFXBallVector lJavaFXBall = new JavaFXBallVector();
			
			// create the JavaFX container to hold the balls and add the first ball
			Group root = new Group();
			root.getChildren().add(lJavaFXBall.img);
			javaFXBalls.add(lJavaFXBall);
			
			// add 99 more balls by cloning it (this is the way the Swing test does it... clone also adds the ball to the group)
			for (int i = 0; i < 99; i++)
			{
				javaFXBalls.add( lJavaFXBall.clone() );
			}
			
			// create scene
			Scene scene = new Scene(root, 500, 300);
			
			// create and show stage
			stage.setTitle("Bubblemark");
			stage.setScene(scene);
			stage.show();
			
			// create the incrementing test scenarion
			// let's be bold and time using nanoseconds; expectations are high
			starttimeNano = System.nanoTime();
			new AnimationTimer()
			{
				@Override
				public void handle(long arg0)
				{
					// for all balls
					int s = javaFXBalls.size();
					for (int i = 0; i < s; i++)
					{
						// move the ball
						javaFXBalls.get(i).move();
					}
					
					// for all possible ball interactions
					for (int i = 0; i < s; i++)
					{
						for (int j = i+1; j < s; j++)
						{
							// detect collision and change movement accordingly
							javaFXBalls.get(i).doCollide( javaFXBalls.get(j) );
						}
					}
					
					// count the frame
					frameCnt++;
					
					// check if a second has passed
					long currenttimeNano = System.nanoTime();
					if (currenttimeNano > lasttimeFPS + 1000000000)
					{
						// print out each FPS on stdout
						System.out.println(javaFXBalls.size() + " balls " + frameCnt + " fps");

						// increase the test counter
						measurementTest++;
						
						// after 5 tests (warm up period) 
						if (measurementTest == 5)
						{
							// print the result on stderr in a Excel readable form
							System.err.println(javaFXBalls.size() + ";" + frameCnt + ";");
							
							// add 100 balls
							for (int i = 0; i < 100; i++)
							{
								javaFXBalls.add( lJavaFXBall.clone() );
							}
							
							// clear the test counter
							measurementTest = 0;
							
							// stop at 5000 balls
							if (javaFXBalls.size() > 5000) System.exit(0);
						}
						
						// reset frame count and time
						frameCnt = 0;
						lasttimeFPS = currenttimeNano;
					}
				}
			}.start();
		}
		catch (Throwable t) { t.printStackTrace(); }
	}
	List<JavaFXBall> javaFXBalls = new ArrayList<JavaFXBall>();
	int frameCnt = 0;
	long starttimeNano = 0;
	long lasttimeFPS = 0;
	int measurementTest = 0;
}
